<?php

namespace Pinpoint\ProductPersonalisation\Model\Pdf;

/**
 * Class Generation
 * @package Pinpoint\ProductPersonalisation\Model\Pdf
 */
class Generation extends \Magento\Framework\Model\AbstractModel
{
    /**
     * @var int
     */
    protected $pageHeight = 80;

    /**
     * @var int
     */
    protected $pageWidth = 80;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageTemplate
     */
    protected $imageTemplate;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageRow
     */
    protected $imageRow;

    /**
     * @var \Pinpoint\Feefo\Helper\Data
     */
    protected $personalisationHelper;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\Font
     */
    protected $productFont;

    /**
     * @var \Zend_Pdf
     */
    protected $pdf;

    /**
     * @var \Magento\Framework\Filesystem
     */
    protected $fileSystem;

    const TEXT_ALIGN_LEFT = 'left';

    const TEXT_ALIGN_RIGHT = 'right';

    const TEXT_ALIGN_CENTER = 'center';

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Pinpoint\ProductPersonalisation\Model\ImageTemplate $imageTemplate,
        \Pinpoint\ProductPersonalisation\Model\ImageRow $imageRow,
        \Pinpoint\ProductPersonalisation\Helper\Data $personalisationHelper,
        \Pinpoint\ProductPersonalisation\Model\Font $productFont,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->imageTemplate = $imageTemplate;
        $this->imageRow = $imageRow;
        $this->personalisationHelper = $personalisationHelper;
        $this->productFont = $productFont;
        $this->fileSystem = $filesystem;
        $this->pdf = new \Zend_Pdf;

        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * @param $item
     * @return \Zend_Pdf
     */
    public function generatePrintRead($item)
    {
        $options = $item->getProductOptions()['options'];
        $imageTemplate = $this->imageTemplate
            ->getCollection()
            ->addFieldToFilter('template_id', $item->getSku())
            ->addFieldToFilter('row_count', $this->getImageRowCount($options))
            ->getFirstItem()
        ;

        $this->addPage((int) $imageTemplate->getData('outer_canvas'), (int) $imageTemplate->getData('outer_width'));

        if ($imageTemplate) {
            foreach ($options as $option) {
                if ($option['option_type'] == 'field') {
                    $optionLabel = $this->personalisationHelper->getCurrentTemplateRowName($option, $imageTemplate->getId());
                    $row = $this->imageRow
                        ->getCollection()
                        ->addFieldToFilter('parent_template_id', $imageTemplate->getId())
                        ->addFieldToFilter('row_label', $optionLabel)
                        ->getFirstItem()
                    ;

                    $this->setFont(
                        \Zend_Pdf_Font::fontWithPath(
                            $this->getFontPath($this->getChosenFont($item->getProductOptions()))
                        ),
                        $row->getData('font_size')
                    );

                    $this->writeText(
                        $option['value'],
                        $row
                    );

                    continue;
                }
            }
        }

        return $this->pdf;
    }

    /**
     * @param null $height
     * @param null $width
     * @return $this
     */
    protected function addPage($height = null, $width = null)
    {
        if ($height && $width) {
            $this->pageHeight = $height;
            $this->pageWidth = $width;
        }

        $this->pdf->pages[] = new \Zend_Pdf_Page($this->getPageSize());

        return $this;
    }

    /**
     * @param $fontName
     * @return string
     */
    protected function getFontPath($fontName)
    {
        $mediaPath = $this->fileSystem
            ->getDirectoryRead(\Magento\Framework\App\Filesystem\DirectoryList::MEDIA)
            ->getAbsolutePath();

        $font = $this->productFont->load($fontName, 'font_name');

        return $mediaPath
            . \Pinpoint\ProductPersonalisation\Model\Font::MEDIA_DIR
            . DIRECTORY_SEPARATOR . $font->getData('font_file')
        ;
    }

    /**
     * @param $font
     * @param $size
     * @param int $page
     * @return $this
     */
    protected function setFont($font, $size, $page = 0)
    {
        $this->pdf->pages[$page]->setFont($font, $size); //($size * 96 / 72));

        return $this;
    }

    /**
     * @param $text
     * @param $row
     * @param int $page
     * @return $this
     */
    protected function writeText($text, $row, $page = 0)
    {
        $pdfPage = $this->pdf->pages[$page];
        $x = $row->getData('left');
        $y = $row->getData('top') + $row->getData('height');
        $align = constant('self::TEXT_ALIGN_' . strtoupper($row->getData('text_alignment')));
        switch ($align) {
            case self::TEXT_ALIGN_RIGHT:
                $x -= $this->getTextWidth($text, $pdfPage->getFont(), $pdfPage->getFontSize());
                break;
            case self::TEXT_ALIGN_CENTER:
                $text_width = $this->getTextWidth($text, $pdfPage->getFont(), $pdfPage->getFontSize());
                $box_width = $row->getData('width') + $row->getData('left');
                $x = $x + (($box_width / 2) - ($text_width / 2));
                $x = $x + ($pdfPage->getWidth() / 2) / 2;
                break;
            default:
                break;
        }

        $this->pdf->pages[$page]->drawText($text, $x, $this->pageHeight - $y);
        return $this;
    }

    /**
     * @param $text
     * @param \Zend_Pdf_Resource_Font $font
     * @param $font_size
     * @return float|int
     */
    protected function getTextWidth($text, \Zend_Pdf_Resource_Font $font, $font_size)
    {
        $drawing_text = iconv('', 'UTF-8', $text);
        $characters    = array();
        for ($i = 0; $i < strlen($drawing_text); $i++) {
            $characters[] = (ord($drawing_text[$i++]) << 8) | ord ($drawing_text[$i]);
        }
        $glyphs        = $font->glyphNumbersForCharacters($characters);
        $widths        = $font->widthsForGlyphs($glyphs);
        $text_width   = (array_sum($widths) / $font->getUnitsPerEm()) * $font_size;
        return ($text_width * (3/4)) + $text_width;
    }

    /**
     * @return string
     */
    protected function getPageSize()
    {
        return sprintf("%s:%s", $this->pageWidth, $this->pageHeight);
    }

    /**
     * @param $options
     * @return int
     */
    protected function getImageRowCount($options)
    {
        $count = 0;
        foreach ($options as $option) {
            if (substr($option['label'], 0, 5) == 'Line ') {
                $count++;
            }
        }

        return $count;
    }

    /**
     * @param $options
     * @return null|string
     */
    protected function getChosenFont($options)
    {
        foreach ($options['options'] as $option) {
            if ($option['label'] == 'Choose Font') {
                return $option['value'];
            }
        }

        return null;
    }
}